# Generated by roxygen2: do not edit by hand

S3method(nplot,default)
S3method(nplot,igraph)
S3method(nplot,matrix)
S3method(nplot,network)
S3method(print,netplot)
S3method(print,netplot_legend)
export(alter)
export(colorRamp2)
export(color_formula)
export(colorkey)
export(ego)
export(get_edge_gpar)
export(get_gpar)
export(get_vertex_gpar)
export(locate_vertex)
export(make_colors)
export(make_edges_colors)
export(make_vertex_colors)
export(nplot)
export(nplot_base)
export(nplot_legend)
export(npolygon)
export(piechart)
export(segments_gradient)
export(set_edge_gpar)
export(set_gpar)
export(set_vertex_gpar)
import(grid)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,rgb)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(igraph,degree)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(igraph,vcount)
importFrom(network,as.edgelist)
importFrom(sna,gplot.layout.kamadakawai)
importFrom(stats,quantile)
importFrom(stats,terms)
